package com.biotechvana.netools.internal;

import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.workbench.modeling.EPartService;
import org.eclipse.jface.action.IStatusLineManager;
import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.jface.resource.JFaceResources;
import org.eclipse.swt.SWT;
import org.eclipse.swt.browser.Browser;
import org.eclipse.swt.browser.BrowserFunction;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.json.JSONArray;
import org.json.JSONObject;

import com.biotechvana.netools.INetworkView;
import com.biotechvana.netools.models.Edge;
import com.biotechvana.netools.models.NetworkModel;
import com.biotechvana.netools.models.Node;
import com.fasterxml.jackson.core.JsonProcessingException;

import org.eclipse.swt.browser.ProgressEvent;
import org.eclipse.swt.browser.ProgressListener;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.events.MenuAdapter;
import org.eclipse.swt.events.MenuEvent;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.MenuItem;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import jakarta.annotation.PostConstruct;
import jakarta.inject.Inject;

import java.io.File;
import java.util.HashSet;
import java.util.Set;

public class NetworkView implements INetworkView  {
	private static final Logger logger = LoggerFactory.getLogger(NetworkView.class);

	Browser browser;
	NetworkModel network;


	@Inject
	IStatusLineManager manager;

	@Inject
	UISynchronize uiSync;

	boolean isDirty = false;


	// for debugging
	private boolean allowInspect = false;



	// some layout properties
	boolean showNavigation = false;
	boolean showLabels = true;
	boolean showEdges = true;
	boolean usePhysics = true;
	boolean fixX=false;
	boolean fixY=false;
	boolean smoothCurves = false;
	boolean hideEdgesOnDrag = true;

	@PostConstruct
	public void createPartControl(Composite parent, EPartService partService , MApplication application) {
		logger.info("Creating NetworkView");

		application.getContext().set(INetworkView.class, this);

		Composite container = new Composite(parent, SWT.NONE);
		container.setLayout(new GridLayout(1, false));

		browser = new Browser(container, SWT.NONE);
		browser.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		//
		// Disable the default context menu in the browser
		if (!allowInspect) {
			browser.addListener(SWT.MenuDetect, event -> event.doit = false);
			//createContextMenu();
		}
		//
		//        // Create a custom context menu for the SWT control



		// Add a progress listener to track page loading
		browser.addProgressListener(new ProgressListener() {



			@Override
			public void completed(ProgressEvent event) {
				System.out.println("Page loaded: " + browser.getUrl());
				if (network != null) {
					initGraph(network);
				} else {
					System.out.println("Network is null");
				}


			}

			@Override
			public void changed(ProgressEvent event) {
				// Handle load progress here if needed
				System.out.println("Page changed: " + browser.getUrl());
				//HtmlLoader.load( browser, "NetworkViewer.html" );

			}
		});


		// Load the HTML file
		//browser.setUrl("/home/data/eclipse/git-J11/netools/com.biotechvana.netools/bundles/com.biotechvana.netools.ui/src/com/biotechvana/netools/internal/resources/NetworkViewer.html");
		HtmlLoader.load( browser, "NetworkViewer.html" );
		addBrowserFunction();

		//		browser.addMouseListener(new MouseAdapter() {
		//		@Override
		//			public void mouseDown(MouseEvent e) {
		//				System.out.println("Mouse Down");
		//			}
		//		@Override
		//			public void mouseDoubleClick(MouseEvent e) {
		//				// TODO Auto-generated method stub
		//				super.mouseDoubleClick(e);
		//				
		//			}
		//			
		//		});



		Composite toolbox = new Composite(container, SWT.NONE);
		toolbox.setLayout(new FormLayout());
		GridData gd= new GridData(SWT.FILL, SWT.FILL, true, false);

		toolbox.setLayoutData(gd);
		createToolbox(toolbox);
	}


	private void createContextMenu() {
		Menu contextMenu = new Menu(browser.getShell(), SWT.POP_UP);
		MenuItem menuItem1 = new MenuItem(contextMenu, SWT.PUSH);
		menuItem1.setText("Custom Action 1");
		menuItem1.addListener(SWT.Selection, e -> {
			System.out.println("Custom Action 1 selected");
			// Add your custom action here
		});

		MenuItem menuItem2 = new MenuItem(contextMenu, SWT.PUSH);
		menuItem2.setText("Custom Action 2");
		menuItem2.addListener(SWT.Selection, e -> {
			System.out.println("Custom Action 2 selected");
			// Add your custom action here
		});

		// Attach the custom context menu to the browser
		browser.setMenu(contextMenu);

		// Optionally, you can show the context menu on right-click
		browser.addListener(SWT.MouseDown, event -> {
			if (event.button == 3) { // Right-click
				contextMenu.setVisible(true);
			}
		});

	}


	private void createToolbox(Composite toolbox) {
		int fixedHight = (int) (TRIM_DEFAULT_HEIGHT*0.9);
		// layoutGroup.setText("Layout");

		// Formlayoutdata 
		FormData fromData = new FormData();
		fromData.top = new FormAttachment(0, 0);
		fromData.height = fixedHight;


		// Checkbox for disabling physics
		Button physics = new Button(toolbox, SWT.TOGGLE);
		physics.setLayoutData(fromData);
		physics.setToolTipText("Enable/Disable Physics");
		// physics.setText("Physics");
		// set icon
		physics.setImage(JFaceResources.getImageRegistry().get(Dialog.DLG_IMG_MESSAGE_INFO));
		physics.setSelection(true);
		physics.setSelection(usePhysics);
		physics.addSelectionListener(new SelectionAdapter() {

			@Override
			public void widgetSelected(SelectionEvent e) {
				// Call JavaScript function to enable/disable physics
				usePhysics = physics.getSelection();
				tooglePhysics(physics.getSelection());
			}


		});

		// fix x btn
		Button fixXBtn = new Button(toolbox, SWT.TOGGLE);
		fromData = new FormData();
		fromData.top = new FormAttachment(0, 0);
		fromData.left = new FormAttachment(physics, 5);
		fromData.height = fixedHight;
		fixXBtn.setLayoutData(fromData);
		fixXBtn.setText("FixX");
		fixXBtn.setToolTipText("Fix X axis");
		fixXBtn.setSelection(fixX);
		fixXBtn.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				// Call JavaScript function to enable/disable physics
				fixX = fixXBtn.getSelection();
				//updateFixXY();
				updateFixX();

			}
		});
		// fix y btn	
		Button fixYBtn = new Button(toolbox, SWT.TOGGLE);
		fromData = new FormData();
		fromData.top = new FormAttachment(0, 0);
		fromData.left = new FormAttachment(fixXBtn, 5);
		fromData.height = fixedHight;
		fixYBtn.setLayoutData(fromData);
		fixYBtn.setText("FixY");
		fixYBtn.setToolTipText("Fix Y axis");
		fixYBtn.setSelection(fixY);
		fixYBtn.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				// Call JavaScript function to enable/disable physics
				fixX = fixYBtn.getSelection();
				//updateFixXY();
				updateFixY();
			}
		});

		// checkbox for showing labels

		// checkbox for showing edges


		// show navigation btn
		Button showNavigationBtn = new Button(toolbox, SWT.TOGGLE);
		fromData = new FormData();
		// align to right
		fromData.top = new FormAttachment(0, 0);
		fromData.right = new FormAttachment(100, 0);
		fromData.height = fixedHight;
		showNavigationBtn.setLayoutData(fromData);
		showNavigationBtn.setText("Show Navigation");
		showNavigationBtn.setToolTipText("Show Navigation");
		showNavigationBtn.addSelectionListener(new SelectionAdapter() {
			@Override
			public void widgetSelected(SelectionEvent e) {
				// Call JavaScript function to show/hide navigation
				showNavigation = showNavigationBtn.getSelection();
				showNavigation(showNavigationBtn.getSelection());
			}
		});

	}


	protected void updateFixXY() {
		//		JSONObject options = new JSONObject();
		//		JSONObject nodes = new JSONObject();
		//		JSONObject fixed = new JSONObject();
		//		fixed.put("x", fixX);
		//		fixed.put("y", fixY);
		//		nodes.put("fixed", fixed);
		//		options.put("nodes", nodes);
		//		browser.execute("updateOptions(" + options.toString() + ");");
		browser.execute("updateFixXY(" + fixX + "," + fixY + ");");

	}

	protected void updateFixX() {
		browser.execute("updateFixX(" + fixX + ");");
	}

	protected void updateFixY() {
		browser.execute("updateFixY(" + fixY + ");");
	}

	protected void showNavigation(boolean selection) {
		browser.execute("toggleNavigation(" + selection + ");");

	}


	@Inject
	private void initGraph(@Optional NetworkModel network) {
		if (network == null) {
			System.out.println("Network is null");
			return;
		}
		this.network = network;
		//		// Initialize the network data
		//        NetworkModel network = new NetworkModel();


		// Serialize network to JSON and pass it to JavaScript
		try {
			String networkJson = getNetworkJson();
			uiSync.asyncExec(() -> {
				browser.execute("initializeNetwork(" + networkJson + ");");
			});

		} catch (Exception e) {
			e.printStackTrace();
		}

	}




	private String getNetworkJson() {
		if (network == null) 
			return "";

		JSONObject data = this.network.getJsonData();


		// add init values for layout and physics 
		//		const options = {
		//				  "nodes": {
		//				    "fixed": {
		//				      "x": true		
		//	      "y": true				

		//				    },
		//				    "size": null
		//				  },
		//				  "edges": {
		//				    "smooth": {
		//				      "forceDirection": "none"
		//				    }
		//				  },
		//				  "interaction": {
		//				    "hideEdgesOnDrag": true
		//				  },
		//				  "physics": {
		//				    "enabled": false,
		//				    "minVelocity": 0.75
		//				  }
		//				}
		JSONObject options = new JSONObject();
		JSONObject nodes = new JSONObject();
		JSONObject fixed = new JSONObject();
		fixed.put("x", fixX);
		fixed.put("y", fixY);
		nodes.put("fixed", fixed);
		options.put("nodes", nodes);
		JSONObject edges = new JSONObject();

		if (smoothCurves) {
			JSONObject smooth = new JSONObject();
			smooth.put("type", "dynamic");
			edges.put("smooth", smooth);
		}
		else {
			edges.put("smooth", false);
		}



		options.put("edges", edges);
		JSONObject interaction = new JSONObject();
		interaction.put("hideEdgesOnDrag", hideEdgesOnDrag);
		options.put("interaction", interaction);

		JSONObject physics = new JSONObject();
		physics.put("enabled", usePhysics);
		//physics.put("minVelocity", 0.75);
		options.put("physics", physics);
		data.put("options", options);

		return data.toString();
	}


	private void tooglePhysics(boolean selection) {
		browser.execute("togglePhysics(" + selection + ");");

	}







	IProgressMonitor currentMonitor = null;
	private void addBrowserFunction() {
		// Register BrowserFunction for each event
		new BrowserFunction(browser, "onClick") {
			@Override
			public Object function(Object[] arguments) {
				logger.debug("Click event received in Java with arguments: " + arguments[0]);
				// System.out.println("Click event received in Java with arguments: " + arguments[0]);
				return null;
			}
		};

		new BrowserFunction(browser, "onDoubleClick") {
			@Override
			public Object function(Object[] arguments) {
				// System.out.println("Double click event received in Java with arguments: " + arguments[0]);
				logger.debug("Double click event received in Java with arguments: " + arguments[0]);
				return null;
			}
		};

		new BrowserFunction(browser, "onContext") {
			@Override
			public Object function(Object[] arguments) {
				// System.out.println("Right-click (context) event received in Java with arguments: " + arguments[0]);
				logger.debug("Right-click (context) event received in Java with arguments: " + arguments[0]);
				return null;
			}
		}; 

		new BrowserFunction(browser, "onHold") {
			@Override
			public Object function(Object[] arguments) {
				// System.out.println("Hold event received in Java with arguments: " + arguments[0]);
				logger.debug("Hold event received in Java with arguments: " + arguments[0]);
				return null;
			}
		};

		new BrowserFunction(browser, "onRelease") {
			@Override
			public Object function(Object[] arguments) {
				System.out.println("Release event received in Java with arguments: " + arguments[0]);
				return null;
			}
		};

		new BrowserFunction(browser, "onSelectNode") {
			@Override
			public Object function(Object[] arguments) {
				// System.out.println("Node selected with arguments: " + arguments[0]);
				logger.debug("Node selected with arguments: " + arguments[0]);
				// cast to string[]
				Object[] nodes = (Object[]) arguments[0];
				Set<String> nodesSelected = new HashSet<>();
				for (Object node : nodes) {
					nodesSelected.add((String) node);
				}
				// update a selection manager
				// TODO update selection manager

				return null;
			}
		};

		new BrowserFunction(browser, "onSelectEdge") {
			@Override
			public Object function(Object[] arguments) {
				// System.out.println("Edge selected with arguments: " + arguments[0]);
				logger.debug("Edge selected with arguments: " + arguments[0]);
				return null;
			}
		};

		new BrowserFunction(browser, "onDeselectNode") {
			@Override
			public Object function(Object[] arguments) {
				// System.out.println("Node deselected with arguments: " + arguments[0]);
				logger.debug("Node deselected with arguments: " + arguments[0]);
				return null;
			}
		};

		new BrowserFunction(browser, "onDeselectEdge") {
			@Override
			public Object function(Object[] arguments) {
				// System.out.println("Edge deselected with arguments: " + arguments[0]);
				logger.debug("Edge deselected with arguments: " + arguments[0]);
				return null;
			}
		};

		new BrowserFunction(browser, "onDragStart") {
			@Override
			public Object function(Object[] arguments) {
				System.out.println("Drag start event received in Java with arguments: " + arguments[0]);
				return null;
			}
		};

		new BrowserFunction(browser, "onDragging") {
			@Override
			public Object function(Object[] arguments) {
				// System.out.println("Dragging event received in Java with arguments: " + arguments[0]);
				logger.debug("Dragging event received in Java with arguments: " + arguments[0]);
				return null;
			}
		};

		new BrowserFunction(browser, "onDragEnd") {
			@Override
			public Object function(Object[] arguments) {
				System.out.println("Drag end event received in Java with arguments: " + arguments[0]);
				return null;
			}
		};

		new BrowserFunction(browser, "onHoverNode") {
			@Override
			public Object function(Object[] arguments) {
				System.out.println("Hover node event received in Java with arguments: " + arguments[0]);
				return null;
			}
		};

		new BrowserFunction(browser, "onBlurNode") {
			@Override
			public Object function(Object[] arguments) {
				System.out.println("Blur node event received in Java with arguments: " + arguments[0]);
				return null;
			}
		};

		new BrowserFunction(browser, "onHoverEdge") {
			@Override
			public Object function(Object[] arguments) {
				System.out.println("Hover edge event received in Java with arguments: " + arguments[0]);
				return null;
			}
		};

		new BrowserFunction(browser, "onBlurEdge") {
			@Override
			public Object function(Object[] arguments) {
				System.out.println("Blur edge event received in Java with arguments: " + arguments[0]);
				return null;
			}
		};

		new BrowserFunction(browser, "onZoom") {
			@Override
			public Object function(Object[] arguments) {
				System.out.println("Zoom event received in Java with arguments: " + arguments[0]);
				return null;
			}
		};

		new BrowserFunction(browser, "onShowPopup") {
			@Override
			public Object function(Object[] arguments) {
				System.out.println("Show popup event received in Java with arguments: " + arguments[0]);
				return null;
			}
		};

		new BrowserFunction(browser, "onHidePopup") {
			@Override
			public Object function(Object[] arguments) {
				System.out.println("Hide popup event received in Java");
				return null;
			}
		};

		new BrowserFunction(browser, "onStabilizationStart") {
			@Override
			public Object function(Object[] arguments) {
				//System.out.println("Stabilization start event received in Java");
				logger.debug("Stabilization start event received in Java");
				currentMonitor = manager.getProgressMonitor();
				currentMonitor.beginTask("Updating", IProgressMonitor.UNKNOWN);
				return null;
			}
		};

		new BrowserFunction(browser, "onStabilizationIterationsDone") {
			@Override
			public Object function(Object[] arguments) {
				System.out.println(
						"Stabilization iterations done event received in Java with arguments: " + arguments[0]);
				return null;
			}
		};

		new BrowserFunction(browser, "onStabilized") {
			@Override
			public Object function(Object[] arguments) {
				System.out.println("Stabilized event received in Java with arguments: " + arguments[0]);
				if (arguments.length > 0 && arguments[0] instanceof String) {
					String jsonString = (String) arguments[0];
					JSONObject jsonObject = new JSONObject(jsonString);
					JSONArray nodePositions = jsonObject.getJSONArray("nodePositions");

					// Start the update in a background thread

					new Thread(new Runnable() {
						@Override
						public void run() {
							// Update node positions in the network
							synchronized (network) {
								for (int i = 0; i < nodePositions.length(); i++) {
									JSONObject nodePosition = nodePositions.getJSONObject(i);
									String nodeId = nodePosition.getString("id");
									double x = nodePosition.getDouble("x");
									double y = nodePosition.getDouble("y");

									Node node = network.getNodeById(nodeId);
									if (node != null) {
										node.setX(x);
										node.setY(y);
										isDirty = true;
									}
								}

								// Notify that the update is done
								network.saveNetwork(currentMonitor);
								if (currentMonitor != null) {
									currentMonitor.done();
								}
							}
						}
					}).start();
				}

				return null;
			}
		};

		new BrowserFunction(browser, "onResize") {
			@Override
			public Object function(Object[] arguments) {
				System.out.println("Resize event received in Java with arguments: " + arguments[0]);
				return null;
			}
		};

		new BrowserFunction(browser, "onInitRedraw") {
			@Override
			public Object function(Object[] arguments) {
				System.out.println("Init redraw event received in Java");
				return null;
			}
		};

		new BrowserFunction(browser, "onBeforeDrawing") {
			@Override
			public Object function(Object[] arguments) {
				System.out.println("Before drawing event received in Java");
				return null;
			}
		};

		new BrowserFunction(browser, "onAfterDrawing") {
			@Override
			public Object function(Object[] arguments) {
				System.out.println("After drawing event received in Java");
				return null;
			}
		};
	}

	public void setFocus() {
		browser.setFocus();
	}


	/**
	 * Default height for workbench trim.
	 */
	public static final int TRIM_DEFAULT_HEIGHT;
	static {
		Shell s = new Shell(Display.getCurrent(), SWT.NONE);
		s.setLayout(new GridLayout());
		ToolBar t = new ToolBar(s, SWT.NONE);
		t.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
		ToolItem ti = new ToolItem(t, SWT.PUSH);
		ti.setImage(JFaceResources.getImageRegistry().get(Dialog.DLG_IMG_MESSAGE_INFO));
		s.layout();
		int toolItemHeight = t.computeSize(SWT.DEFAULT, SWT.DEFAULT).y;
		GC gc = new GC(s);
		Point fontSize = gc.textExtent("Wg"); //$NON-NLS-1$
		gc.dispose();
		TRIM_DEFAULT_HEIGHT = Math.max(toolItemHeight, fontSize.y);
		s.dispose();

	}
	@Override
	public void saveNetwork() {
		if (network == null) {
			logger.error("NetworkView : No Network  to save");
			return;
		}
		currentMonitor = manager.getProgressMonitor();
		currentMonitor.beginTask("Saving", IProgressMonitor.UNKNOWN);
		new Thread(new Runnable() {
			@Override
			public void run() {
				// Update node positions in the network
				synchronized (network) {
					// Notify that the update is done
					network.saveNetwork(currentMonitor);
					isDirty = false;
					if (currentMonitor != null) {
						currentMonitor.done();
					}
				}
			}
		}).start();

	}

}