package com.biotechvana.netools.internal;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

import org.eclipse.e4.core.di.annotations.Optional;
import org.eclipse.e4.ui.di.UISynchronize;
import org.eclipse.e4.ui.model.application.MApplication;
import org.eclipse.e4.ui.progress.IProgressService;
import org.eclipse.jface.layout.GridDataFactory;
import org.eclipse.nebula.widgets.nattable.NatTable;
import org.eclipse.nebula.widgets.nattable.data.IColumnAccessor;
import org.eclipse.nebula.widgets.nattable.data.IColumnPropertyAccessor;
import org.eclipse.nebula.widgets.nattable.data.IDataProvider;
import org.eclipse.nebula.widgets.nattable.data.ListDataProvider;
import org.eclipse.nebula.widgets.nattable.grid.data.DefaultColumnHeaderDataProvider;
import org.eclipse.nebula.widgets.nattable.grid.data.DefaultCornerDataProvider;
import org.eclipse.nebula.widgets.nattable.grid.data.DefaultRowHeaderDataProvider;
import org.eclipse.nebula.widgets.nattable.grid.layer.ColumnHeaderLayer;
import org.eclipse.nebula.widgets.nattable.grid.layer.CornerLayer;
import org.eclipse.nebula.widgets.nattable.grid.layer.GridLayer;
import org.eclipse.nebula.widgets.nattable.grid.layer.RowHeaderLayer;
import org.eclipse.nebula.widgets.nattable.hideshow.ColumnHideShowLayer;
import org.eclipse.nebula.widgets.nattable.layer.AbstractLayerTransform;
import org.eclipse.nebula.widgets.nattable.layer.DataLayer;
import org.eclipse.nebula.widgets.nattable.reorder.ColumnReorderLayer;
import org.eclipse.nebula.widgets.nattable.selection.SelectionLayer;
import org.eclipse.nebula.widgets.nattable.viewport.ViewportLayer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import com.biotechvana.netools.models.NetworkModel;
import com.biotechvana.netools.models.Node;
import com.biotechvana.netools.projects.IProjectsManager;
import com.biotechvana.netools.projects.Project;

import jakarta.annotation.PostConstruct;
import jakarta.inject.Inject;
import jakarta.inject.Named;

public class NodesViewer {
	public static final Logger logger = LoggerFactory.getLogger(NodesViewer.class);
	
	
	class NodeColumnPropertyAccessor implements IColumnPropertyAccessor<Node> {

		String[] nodeModel;
		Map<String, Integer> columnIndexes = new HashMap<>();
		public NodeColumnPropertyAccessor(String[] nodeModel) {
			this.nodeModel = nodeModel;
			for (int i = 0; i < nodeModel.length; i++) {
				columnIndexes.put(nodeModel[i], i);
			}
		
		}
		
		@Override
		public Object getDataValue(Node rowObject, int columnIndex) {
			String property = nodeModel[columnIndex];
			if (property.equals("id"))
				return rowObject.getId();
			if (property.equals("label"))
				return rowObject.getLabel();
			
			return rowObject.getAttribute(property);
			
		}

		@Override
		public void setDataValue(Node rowObject, int columnIndex, Object newValue) {
			String property = nodeModel[columnIndex];
			if (property.equals("id"))
                return;
              
			rowObject.setAttribute(property, newValue);

		}

		@Override
		public int getColumnCount() {
			
			return nodeModel.length;
		}

		@Override
		public String getColumnProperty(int columnIndex) {
			
			return  nodeModel[columnIndex];
		}

		@Override
		public int getColumnIndex(String propertyName) {
			return columnIndexes.get(propertyName);
		}

	}
	
	
	
	class NodeDataProvider extends ListDataProvider<Node> {

		public NodeDataProvider(List<Node> list, IColumnAccessor<Node> columnAccessor) {
			super(list, columnAccessor);
			// TODO Auto-generated constructor stub
		}

	}
	
	public class BodyLayerStack extends AbstractLayerTransform {

        private SelectionLayer selectionLayer;

        public BodyLayerStack(IDataProvider dataProvider) {
            DataLayer bodyDataLayer = new DataLayer(dataProvider);
            ColumnReorderLayer columnReorderLayer = new ColumnReorderLayer(
                    bodyDataLayer);
            ColumnHideShowLayer columnHideShowLayer = new ColumnHideShowLayer(
                    columnReorderLayer);
            this.selectionLayer = new SelectionLayer(columnHideShowLayer);
            ViewportLayer viewportLayer = new ViewportLayer(this.selectionLayer);
            setUnderlyingLayer(viewportLayer);
        }

        public SelectionLayer getSelectionLayer() {
            return this.selectionLayer;
        }
    }
	public class ColumnHeaderLayerStack extends AbstractLayerTransform {

        public ColumnHeaderLayerStack(IDataProvider dataProvider, BodyLayerStack bodyLayer) {
            DataLayer dataLayer = new DataLayer(dataProvider);
            ColumnHeaderLayer colHeaderLayer = new ColumnHeaderLayer(dataLayer,
                    bodyLayer, bodyLayer.getSelectionLayer());
            setUnderlyingLayer(colHeaderLayer);
        }
    }
	public class RowHeaderLayerStack extends AbstractLayerTransform {

        public RowHeaderLayerStack(IDataProvider dataProvider , BodyLayerStack bodyLayer) {
            DataLayer dataLayer = new DataLayer(dataProvider, 50, 20);
            RowHeaderLayer rowHeaderLayer = new RowHeaderLayer(dataLayer,
                    bodyLayer, bodyLayer.getSelectionLayer());
            setUnderlyingLayer(rowHeaderLayer);
        }
    }
	
	@Inject
	UISynchronize sync;
	Project activeProject;
	IProjectsManager projectsManager;

	@Inject
	IProgressService progressService;
	
	NetworkModel network;
	
	
	@Inject
	@Optional 
	public void setActiveProject(@Named(IProjectsManager.ACTIVE_PROJECT) Project project) {
		activeProject = project;
    
	}
	
	@Inject
	private void setNetworkModel(@Optional NetworkModel network) {
		this.network = network;
		logger.debug("NodesViewer.setNetworkModel");
		if (mainBody != null && !mainBody.isDisposed()) {
			crreateTableViewer(mainBody);
		}
	}
	
	
	Composite mainBody;
	Composite tableComposite;
	
	@PostConstruct
	public void createPartControl(Composite parent, MApplication app, IProjectsManager projectsManager) { 
		this.projectsManager = projectsManager;
        logger.debug("NodesViewer.createPartControl");
        mainBody = new Composite(parent, SWT.NONE);
        mainBody.setLayout(new FillLayout());
        crreateTableViewer(mainBody);
        
	}

	private void crreateTableViewer(Composite parent) {
		logger.debug("NodesViewer.crreateTableViewer");
		if (tableComposite != null && !tableComposite.isDisposed()) {
			tableComposite.dispose();
		}
		tableComposite = new Composite(parent, SWT.NONE);
        tableComposite.setLayout(new GridLayout());
        createNatTable(tableComposite);
		
	}

	private void createNatTable(Composite tableComposite2) {
		if (network == null) {
			return;
		}
		
		 Set<String> modelHeader = network.getNodeModel();
		 List<Node> nodeList =  List.copyOf(network.getNodes());
		 
		 String[] propertyNames = new String[modelHeader.size()];
		 Map<String, String> propertyToLabels = modelHeader.stream().collect(Collectors.toMap(h -> h, h -> {
			 String label = h;
			 // set first letter to upper case
			 label = label.substring(0, 1).toUpperCase() + label.substring(1);
			 return label;
			 }));
		 // first id
		 propertyNames[0] = "id";
		 modelHeader.remove("id");
		 // second label
		 propertyNames[1] = "label";
		 modelHeader.remove("label");
		 int i = 2;
		 for (String property : modelHeader) {
             propertyNames[i] = property;
             i++;
          }
		 

		 IColumnPropertyAccessor<Node> columnPropertyAccessor = new NodeColumnPropertyAccessor(propertyNames);
		 
		 IDataProvider bodyDataProvider = new NodeDataProvider(nodeList, columnPropertyAccessor);
		 // DataLayer bodyDataLayer = new DataLayer(bodyDataProvider);
		 DefaultColumnHeaderDataProvider colHeaderDataProvider = new DefaultColumnHeaderDataProvider(
                 propertyNames, propertyToLabels);
         DefaultRowHeaderDataProvider rowHeaderDataProvider = new DefaultRowHeaderDataProvider(
                bodyDataProvider);
         BodyLayerStack bodyLayer = new BodyLayerStack(bodyDataProvider);
         ColumnHeaderLayerStack columnHeaderLayer = new ColumnHeaderLayerStack(
                 colHeaderDataProvider, bodyLayer);
         RowHeaderLayerStack rowHeaderLayer = new RowHeaderLayerStack(
                 rowHeaderDataProvider,bodyLayer);
         DefaultCornerDataProvider cornerDataProvider = new DefaultCornerDataProvider(
                 colHeaderDataProvider, rowHeaderDataProvider);
         CornerLayer cornerLayer = new CornerLayer(new DataLayer(
                 cornerDataProvider), rowHeaderLayer, columnHeaderLayer);

         
         
         GridLayer gridLayer = new GridLayer(bodyLayer, columnHeaderLayer,
                 rowHeaderLayer, cornerLayer);
		 NatTable natTable = new NatTable(tableComposite2, gridLayer);
		 GridDataFactory.fillDefaults().grab(true, true).applyTo(natTable);
		 mainBody.layout(true);
	}
}
