package com.biotechvana.netools.ui.editors.internal;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.observable.value.WritableValue;
import org.eclipse.jface.databinding.swt.typed.WidgetProperties;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.ScrolledComposite;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.ColorDialog;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Spinner;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.forms.widgets.FormToolkit;
import org.eclipse.ui.forms.widgets.Section;

import com.biotechvana.netools.models.INode;
import com.biotechvana.netools.models.Node;
import com.biotechvana.netools.ui.editors.INetworkEditor;
import com.biotechvana.netools.ui.editors.INodeEditor;

import jakarta.annotation.PostConstruct;

public class NodeEditor implements INodeEditor {

    private INetworkEditor networkEditor;

    private DataBindingContext bindingContext;
    private WritableValue<Node> selectedNode;
    private Text idText;
    private Spinner borderWidthSpinner;
    private Spinner borderWidthSelectedSpinner;
    private Text opacityText;
    private Text groupText;
    private Text hiddenText;
    private Text labelText;
    private Text labelHighlightBoldText;
    private Spinner levelSpinner;
    private Text massText;
    private Text shapeText;
    private Text physicsText;
    private Spinner sizeSpinner;
    private Text titleText;
    private Text valueText;
    private Text xText;
    private Text yText;
    private Text colorText;
    private Text colorBorderText;
    private Text colorHighlightText;
    private Text colorHighlightBorderText;
    private Text colorHoverText;
    private Text colorHoverBorderText;

    @PostConstruct
    public void initEditor(Composite parent) {
        FormToolkit toolkit = new FormToolkit(parent.getDisplay());
        this.bindingContext = new DataBindingContext();
        this.selectedNode = new WritableValue<>(null, Node.class);

        ScrolledComposite scrolledComposite = new ScrolledComposite(parent, SWT.H_SCROLL | SWT.V_SCROLL);
        scrolledComposite.setExpandHorizontal(true);
        scrolledComposite.setExpandVertical(true);

        Composite editorComposite = toolkit.createComposite(scrolledComposite);
        editorComposite.setLayout(new GridLayout(1, false));
        editorComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true));
        ((GridLayout) editorComposite.getLayout()).marginWidth = 10;
        ((GridLayout) editorComposite.getLayout()).marginHeight = 10;

        // General Section
        Section generalSection = toolkit.createSection(editorComposite, Section.TITLE_BAR | Section.TWISTIE | Section.EXPANDED);
        generalSection.setText("General");
        generalSection.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
        Composite generalComposite = toolkit.createComposite(generalSection);
        generalComposite.setLayout(new GridLayout(2, false));
        generalSection.setClient(generalComposite);

        // ID
        toolkit.createLabel(generalComposite, "ID:");
        idText = toolkit.createText(generalComposite, "", SWT.BORDER);
        idText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

        // Label
        toolkit.createLabel(generalComposite, "Label:");
        labelText = toolkit.createText(generalComposite, "", SWT.BORDER);
        labelText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

        // Group
        toolkit.createLabel(generalComposite, "Group:");
        groupText = toolkit.createText(generalComposite, "", SWT.BORDER);
        groupText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

        // Appearance Section
        Section appearanceSection = toolkit.createSection(editorComposite, Section.TITLE_BAR | Section.TWISTIE | Section.EXPANDED);
        appearanceSection.setText("Appearance");
        appearanceSection.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
        Composite appearanceComposite = toolkit.createComposite(appearanceSection);
        appearanceComposite.setLayout(new GridLayout(3, false)); // Adjusted to 3 columns for color buttons
        appearanceSection.setClient(appearanceComposite);

        // Border Width
        toolkit.createLabel(appearanceComposite, "Border Width:");
        borderWidthSpinner = new Spinner(appearanceComposite, SWT.BORDER);
        borderWidthSpinner.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));
        toolkit.adapt(borderWidthSpinner);

        // Border Width Selected
        toolkit.createLabel(appearanceComposite, "Border Width Selected:");
        borderWidthSelectedSpinner = new Spinner(appearanceComposite, SWT.BORDER);
        borderWidthSelectedSpinner.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false,2, 1));
        toolkit.adapt(borderWidthSelectedSpinner);

        // Opacity
        toolkit.createLabel(appearanceComposite, "Opacity:");
        opacityText = toolkit.createText(appearanceComposite, "", SWT.BORDER);
        opacityText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 2, 1));

        // Color
        toolkit.createLabel(appearanceComposite, "Color:");
        colorText = toolkit.createText(appearanceComposite, "", SWT.BORDER);
        colorText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
        Button colorButton = toolkit.createButton(appearanceComposite, "Choose...", SWT.PUSH);
        colorButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                ColorDialog colorDialog = new ColorDialog(parent.getShell());
                RGB rgb = colorDialog.open();
                if (rgb != null) {
                    String colorHex = String.format("#%02x%02x%02x", rgb.red, rgb.green, rgb.blue);
                    colorText.setText(colorHex);
                    colorText.setBackground(new Color(parent.getDisplay(), rgb));
                }
            }
        });

        // Color Border
        toolkit.createLabel(appearanceComposite, "Color Border:");
        colorBorderText = toolkit.createText(appearanceComposite, "", SWT.BORDER);
        colorBorderText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
        Button colorBorderButton = toolkit.createButton(appearanceComposite, "Choose...", SWT.PUSH);
        colorBorderButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                ColorDialog colorDialog = new ColorDialog(parent.getShell());
                RGB rgb = colorDialog.open();
                if (rgb != null) {
                    String colorHex = String.format("#%02x%02x%02x", rgb.red, rgb.green, rgb.blue);
                    colorBorderText.setText(colorHex);
                    colorBorderText.setBackground(new Color(parent.getDisplay(), rgb));
                }
            }
        });

        // Color Highlight
        toolkit.createLabel(appearanceComposite, "Color Highlight:");
        colorHighlightText = toolkit.createText(appearanceComposite, "", SWT.BORDER);
        colorHighlightText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
        Button colorHighlightButton = toolkit.createButton(appearanceComposite, "Choose...", SWT.PUSH);
        colorHighlightButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                ColorDialog colorDialog = new ColorDialog(parent.getShell());
                RGB rgb = colorDialog.open();
                if (rgb != null) {
                    String colorHex = String.format("#%02x%02x%02x", rgb.red, rgb.green, rgb.blue);
                    colorHighlightText.setText(colorHex);
                    colorHighlightText.setBackground(new Color(parent.getDisplay(), rgb));
                }
            }
        });

        // Color Highlight Border
        toolkit.createLabel(appearanceComposite, "Color Highlight Border:");
        colorHighlightBorderText = toolkit.createText(appearanceComposite, "", SWT.BORDER);
        colorHighlightBorderText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
        Button colorHighlightBorderButton = toolkit.createButton(appearanceComposite, "Choose...", SWT.PUSH);
        colorHighlightBorderButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                ColorDialog colorDialog = new ColorDialog(parent.getShell());
                RGB rgb = colorDialog.open();
                if (rgb != null) {
                    String colorHex = String.format("#%02x%02x%02x", rgb.red, rgb.green, rgb.blue);
                    colorHighlightBorderText.setText(colorHex);
                    colorHighlightBorderText.setBackground(new Color(parent.getDisplay(), rgb));
                }
            }
        });

        // Color Hover
        toolkit.createLabel(appearanceComposite, "Color Hover:");
        colorHoverText = toolkit.createText(appearanceComposite, "", SWT.BORDER);
        colorHoverText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
        Button colorHoverButton = toolkit.createButton(appearanceComposite, "Choose...", SWT.PUSH);
        colorHoverButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                ColorDialog colorDialog = new ColorDialog(parent.getShell());
                RGB rgb = colorDialog.open();
                if (rgb != null) {
                    String colorHex = String.format("#%02x%02x%02x", rgb.red, rgb.green, rgb.blue);
                    colorHoverText.setText(colorHex);
                    colorHoverText.setBackground(new Color(parent.getDisplay(), rgb));
                }
            }
        });

        // Color Hover Border
        toolkit.createLabel(appearanceComposite, "Color Hover Border:");
        colorHoverBorderText = toolkit.createText(appearanceComposite, "", SWT.BORDER);
        colorHoverBorderText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
        Button colorHoverBorderButton = toolkit.createButton(appearanceComposite, "Choose...", SWT.PUSH);
        colorHoverBorderButton.addSelectionListener(new SelectionAdapter() {
            @Override
            public void widgetSelected(SelectionEvent e) {
                ColorDialog colorDialog = new ColorDialog(parent.getShell());
                RGB rgb = colorDialog.open();
                if (rgb != null) {
                    String colorHex = String.format("#%02x%02x%02x", rgb.red, rgb.green, rgb.blue);
                    colorHoverBorderText.setText(colorHex);
                    colorHoverBorderText.setBackground(new Color(parent.getDisplay(), rgb));
                }
            }
        });

        // Position Section
        Section positionSection = toolkit.createSection(editorComposite, Section.TITLE_BAR | Section.TWISTIE | Section.EXPANDED);
        positionSection.setText("Position");
        positionSection.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
        Composite positionComposite = toolkit.createComposite(positionSection);
        positionComposite.setLayout(new GridLayout(2, false));
        positionSection.setClient(positionComposite);

        // X
        toolkit.createLabel(positionComposite, "X:");
        xText = toolkit.createText(positionComposite, "", SWT.BORDER);
        xText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

        // Y
        toolkit.createLabel(positionComposite, "Y:");
        yText = toolkit.createText(positionComposite, "", SWT.BORDER);
        yText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

        // Physics Section
        Section physicsSection = toolkit.createSection(editorComposite, Section.TITLE_BAR | Section.TWISTIE | Section.EXPANDED);
        physicsSection.setText("Physics");
        physicsSection.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, false));
        Composite physicsComposite = toolkit.createComposite(physicsSection);
        physicsComposite.setLayout(new GridLayout(2, false));
        physicsSection.setClient(physicsComposite);

        // Mass
        toolkit.createLabel(physicsComposite, "Mass:");
        massText = toolkit.createText(physicsComposite, "", SWT.BORDER);
        massText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

        // Physics
        toolkit.createLabel(physicsComposite, "Physics:");
        physicsText = toolkit.createText(physicsComposite, "", SWT.BORDER);
        physicsText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

        // Shape
        toolkit.createLabel(physicsComposite, "Shape:");
        shapeText = toolkit.createText(physicsComposite, "", SWT.BORDER);
        shapeText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

        // Size
        toolkit.createLabel(physicsComposite, "Size:");
        sizeSpinner = new Spinner(physicsComposite, SWT.BORDER);
        sizeSpinner.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));
        toolkit.adapt(sizeSpinner);

        // Title
        toolkit.createLabel(physicsComposite, "Title:");
        titleText = toolkit.createText(physicsComposite, "", SWT.BORDER);
        titleText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

        // Value
        toolkit.createLabel(physicsComposite, "Value:");
        valueText = toolkit.createText(physicsComposite, "", SWT.BORDER);
        valueText.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false));

        scrolledComposite.setContent(editorComposite);
        scrolledComposite.setMinSize(editorComposite.computeSize(SWT.DEFAULT, SWT.DEFAULT));
    }

    public void setNode(INode node) {
        idText.setText(node.getId());
        borderWidthSpinner.setSelection(node.getBorderWidth());
        borderWidthSelectedSpinner.setSelection(node.getBorderWidthSelected());
        opacityText.setText(String.valueOf(node.getOpacity()));
        groupText.setText(node.getGroup());
        hiddenText.setText(String.valueOf(node.isHidden()));
        labelText.setText(node.getLabel());
        labelHighlightBoldText.setText(String.valueOf(node.isLabelHighlightBold()));
        levelSpinner.setSelection(node.getLevel());
        massText.setText(String.valueOf(node.getMass()));
        shapeText.setText(node.getShape());
        physicsText.setText(String.valueOf(node.isPhysics()));
        sizeSpinner.setSelection(node.getSize());
        titleText.setText(node.getTitle());
        valueText.setText(String.valueOf(node.getValue()));
        xText.setText(String.valueOf(node.getX()));
        yText.setText(String.valueOf(node.getY()));
        colorText.setText(node.getColor());
        colorBorderText.setText(node.getColorBorder());
        colorHighlightText.setText(node.getColorHighlight());
        colorHighlightBorderText.setText(node.getColorHighlightBorder());
        colorHoverText.setText(node.getColorHover());
        colorHoverBorderText.setText(node.getColorHoverBorder());
    }

    public void updateNode(INode node) {
        node.setId(idText.getText());
        node.setBorderWidth(borderWidthSpinner.getSelection());
        node.setBorderWidthSelected(borderWidthSelectedSpinner.getSelection());
        node.setOpacity(Double.parseDouble(opacityText.getText()));
        node.setGroup(groupText.getText());
        node.setHidden(Boolean.parseBoolean(hiddenText.getText()));
        node.setLabel(labelText.getText());
        node.setLabelHighlightBold(Boolean.parseBoolean(labelHighlightBoldText.getText()));
        node.setLevel(levelSpinner.getSelection());
        node.setMass(Double.parseDouble(massText.getText()));
        node.setShape(shapeText.getText());
        node.setPhysics(Boolean.parseBoolean(physicsText.getText()));
        node.setSize(sizeSpinner.getSelection());
        node.setTitle(titleText.getText());
        node.setValue(Double.parseDouble(valueText.getText()));
        node.setX(Double.parseDouble(xText.getText()));
        node.setY(Double.parseDouble(yText.getText()));
        node.setColor(colorText.getText());
        node.setColorBorder(colorBorderText.getText());
        node.setColorHighlight(colorHighlightText.getText());
        node.setColorHighlightBorder(colorHighlightBorderText.getText());
        node.setColorHover(colorHoverText.getText());
        node.setColorHoverBorder(colorHoverBorderText.getText());
    }
}
